/*
 * GroupLens Common Utilities
 * Copyright © 2011 Regents of the University of Minnesota
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *  * Neither the name of the University of Minnesota nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software has been partly funded by NSF grant IIS 08-08692.
 */
package org.grouplens.common.util;

import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.UUID;

import org.apache.commons.codec.binary.Base64;

public class HashUtil {
    /**
     * Compute the hash of a password and a salt using the SHA-256 algorithm.
     * See {@link #getSaltedPassword(String, String, String)} for more
     * information.
     * 
     * @param password The password
     * @param salt The base-64 encoded salt data
     * @return A base-64 encoded hash of the password and salt
     */
    public static String getSaltedPassword(String password, String salt) {
        return getSaltedPassword(password, salt, "SHA-256");
    }

    /**
     * <p>
     * Compute the hash of a password and a salt using the specified hashing
     * algorithm. The salt is assumed to be base-64 encoded data stored in a
     * String. The password is the String as entered by the user.
     * <tt>hashAlgorithm</tt> is the name of an algorithm suitable for use with
     * {@link MessageDigest}. The returned hash is also base-64 encoded.
     * </p>
     * <p>
     * Password salting or hashing should be used so that true passwords are not
     * stored in databases. Instead store the salt and hashed password generated
     * by this method. When needing to check password validity, lookup the salt
     * and compute the hashed password using the entered password. If it matches
     * the stored hash then the password is correct.
     * </p>
     * 
     * @param password The password
     * @param salt The base-64 encoded salt data
     * @return A base-64 encoded hash of the password and salt
     * @throws RuntimeException if the hash algorithm can't be found
     */
    public static String getSaltedPassword(String password, String salt, String hashAlgorithm) {
        try {
            MessageDigest digest = MessageDigest.getInstance(hashAlgorithm);
            digest.reset();
            
            digest.update(Base64.decodeBase64(salt));
            byte[] hash = digest.digest(password.getBytes("UTF-8"));
            return Base64.encodeBase64String(hash);
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * Convenience method for <code>getUUID(name.getBytes(), "SHA")</code>
     * 
     * @param name The name used to generate the UUID
     * @return A UUID based off of the name
     */
    public static UUID getUUID(String name) {
        return getUUID(name.getBytes(), "SHA");
    }

    /**
     * <p>
     * Create a {@link UUID} based on the given 'name', or sequence of bytes.
     * <tt>hashAlgorithm</tt> is the name of a hash algorithm suitable for use
     * with {@link MessageDigest}. This method performs the same actions as
     * {@link UUID#nameUUIDFromBytes(byte[])} except that the hash algorithm is
     * not hard-coded.
     * </p>
     * <p>
     * Possible values for the hash algorithm include 'MD5', 'SHA', and
     * 'SHA-256'.
     * </p>
     * 
     * @param name The name used to generate the UUID
     * @param hashAlgorithm The name of the hash algorithm
     * @return A UUID based on name
     * @throws RuntimeException if the hash algorithm is unsupported
     */
	public static UUID getUUID(byte[] name, String hashAlgorithm) {
	    // this code is taken from UUID.nameUUIDFromBytes but is modified 
	    // to use any algorithm instead of MD5, which is hardcoded in UUID
	    MessageDigest md;
	    try {
	        md = MessageDigest.getInstance(hashAlgorithm);
	        md.reset();
	    } catch(NoSuchAlgorithmException nsae) {
	        throw new RuntimeException(hashAlgorithm + " unsupported", nsae);
	    }
	    
	    byte[] hashBytes = md.digest(name);
	    hashBytes[6] &= 0x0f; // clear version
	    hashBytes[6] |= 0x30; // set to version 3 (name hashing
	    hashBytes[8] &= 0x3f; // clear variant
	    hashBytes[8] |= 0x80; // set to IETF variant
	    
	    // this code is taken from a private constructor in UUID
	    long msb = 0;
	    long lsb = 0;
	    
	    for (int i = 0; i < 8; i++)
	        msb = (msb << 8) | (hashBytes[i] & 0xff);
	    for (int i = 8; i < 16; i++)
	        lsb = (lsb << 8) | (hashBytes[i] & 0xff);
	    
	    return new UUID(msb, lsb);
	}
}
